@@ -8,7 +8,7 @@ from .errors import (
 )
 from .filesystem import clean_datapackage_name
 from .io_helpers import file_writer, file_reader
-from .proxies import UndefinedInterface
+from .proxies import UndefinedInterface, Proxy
 from .utils import (
     check_name,
     check_suffix,
@@ -110,6 +110,31 @@ class DatapackageBase:
         del self.resources[index]
         del self.data[index]
 
+    def del_resource_group(self, name: str) -> None:
+        """Remove a resource group, and delete its data files, if any."""
+        if self._modified:
+            raise PotentialInconsistency(
+                "Datapackage is modified; save modifications or reload"
+            )
+
+        indices = [
+            i
+            for i, resource in enumerate(self.resources)
+            if resource.get("group") == name
+        ]
+
+        for obj in (obj for i, obj in enumerate(self.resources) if i in indices):
+            try:
+                self.fs.remove(obj["path"])
+            except (KeyError, ResourceNotFound):
+                # Interface has no path
+                pass
+
+        self.resources = [
+            obj for i, obj in enumerate(self.resources) if i not in indices
+        ]
+        self.data = [obj for i, obj in enumerate(self.data) if i not in indices]
+
     def get_resource(self, name_or_index: Union[str, int]) -> (Any, dict):
         """Return data and metadata for ``name_or_index``.
 
@@ -133,9 +158,8 @@ class DatapackageBase:
         except KeyError:
             index = self._get_index(name_or_index)
 
-            if isinstance(self.data[index], partial):
-                obj = self.data[index]()
-                self.data[index] = obj
+            if isinstance(self.data[index], (Proxy, partial)):
+                self.data[index] = self.data[index]()
 
             self._cache[name_or_index] = (self.data[index], self.resources[index])
         return self._cache[name_or_index]
@@ -145,7 +169,7 @@ class DatapackageBase:
 
         All included objects are the same as in the original data package, i.e. no copies are made. No checks are made to ensure consistency with modifications to the original datapackage after the creation of this filtered datapackage.
 
-        This method was introduced to allow for the efficient constuction of matrices; each datapackage can have data for multiple matrices, and we can then create filtered datapackages which exclusively have data for the matrix of interest. As such, they should be considered read-only, though this is not enforced."""
+        This method was introduced to allow for the efficient construction of matrices; each datapackage can have data for multiple matrices, and we can then create filtered datapackages which exclusively have data for the matrix of interest. As such, they should be considered read-only, though this is not enforced."""
         fdp = FilteredDatapackage()
         fdp.metadata = {k: v for k, v in self.metadata.items() if k != "resources"}
         fdp.metadata["resources"] = []
@@ -158,6 +182,54 @@ class DatapackageBase:
             fdp.indexer = self.indexer
         return fdp
 
+    def exclude_resource_group(self, group: str) -> "FilteredDatapackage":
+        """Filter a datapackage to exclude a resource group by group name.
+
+        Use ``filter_by_attribute("group", "group_name")`` to get only this resource group.
+
+        """
+        fdp = FilteredDatapackage()
+        fdp.metadata = {k: v for k, v in self.metadata.items() if k != "resources"}
+        fdp.metadata["resources"] = []
+
+        if hasattr(self, "indexer"):
+            fdp.indexer = self.indexer
+
+        to_include = [
+            i
+            for i, resource in enumerate(self.resources)
+            if resource.get("group") != group
+        ]
+        fdp.data = [o for i, o in enumerate(self.data) if i in to_include]
+        fdp.resources = [o for i, o in enumerate(self.resources) if i in to_include]
+        return fdp
+
+    def exclude_resource_group_kind(
+        self, group: str, kind: str
+    ) -> "FilteredDatapackage":
+        """Filter a datapackage to exclude a certain kinds of resources in a given resource group.
+
+        The intended use case is to do complicated calculation setups, like distributions from one resource group but just the static array from a different resource group in the same datapackage.
+
+        ``group`` and ``kind`` are the group and kind labels of the resource to exclude.
+
+        """
+        fdp = FilteredDatapackage()
+        fdp.metadata = {k: v for k, v in self.metadata.items() if k != "resources"}
+        fdp.metadata["resources"] = []
+
+        if hasattr(self, "indexer"):
+            fdp.indexer = self.indexer
+
+        to_exclude = [
+            i
+            for i, resource in enumerate(self.resources)
+            if resource.get("group") == group and resource.get("kind") == kind
+        ]
+        fdp.data = [o for i, o in enumerate(self.data) if i not in to_exclude]
+        fdp.resources = [o for i, o in enumerate(self.resources) if i not in to_exclude]
+        return fdp
+
 
 class FilteredDatapackage(DatapackageBase):
     """A subset of a datapackage. Used in matrix construction or other data manipulation operations.
@@ -310,8 +382,11 @@ class Datapackage(DatapackageBase):
         name = name or uuid.uuid4().hex
 
         existing_names = {o["name"] for o in self.resources}
+        existing_groups = {o["group"] for o in self.resources if o.get("group")}
         if name in existing_names:
             raise NonUnique("This name already used")
+        if name in existing_groups:
+            raise NonUnique("This group name already used")
 
         return name
 
@@ -324,7 +399,9 @@ class Datapackage(DatapackageBase):
         nrows: Union[int, None] = None,
         **kwargs,
     ) -> None:
-        """Note that this function will sort the arrays produced."""
+        """Create a persistant vector from an iterator. Uses the utility function ``resolve_dict_iterator``.
+
+        This is the **only array creation method which produces sorted arrays**."""
         name = self._prepare_name(name)
         (
             data_array,
@@ -782,7 +859,7 @@ def load_datapackage(
 
     Can load proxies to data instead of the data itself, which can be useful when interacting with large arrays or large packages where only a subset of the data will be accessed.
 
-    Proxies just use `functools.partial` to create a callable function instead of returning the raw data. datapackage access methods (i.e. `.get_resource`) will automatically resolve proxies when needed.
+    Proxies use something similar to `functools.partial` to create a callable class instead of returning the raw data (see https://github.com/brightway-lca/bw_processing/issues/9 for why we can't just use `partial`). datapackage access methods (i.e. `.get_resource`) will automatically resolve proxies when needed.
 
     Args:
 
